/** 
 *   Socket initialisation (AF_INET) for antena position system.
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#define LOCAL_PERROR 1

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#undef PERROR
#if LOCAL_PERROR 
#define PERROR(s) perror(s)
#else 
#define PERROR(s)
#endif


/* code to establish a socket */

int socket_init(int port)
{
	int s;
	int l;
	struct sockaddr_in sa;
	int reuse = 1;

	memset(&sa, 0, sizeof(struct sockaddr_in));  /* clear our address */
	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = htonl(INADDR_ANY);
	sa.sin_port = htons(port);

	/* create socket */
	s = socket(AF_INET, SOCK_STREAM, 0);

	if ( setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
			(char *)&reuse,sizeof(reuse)) ) {
		PERROR("socket_init:setsockopt\n"); 
		return -1;
	}

	if (s < 0) {
		PERROR("socket_init:socket");
		return(-1);
	}
	l = sizeof(sa);
	/* bind address to socket */
	if (bind(s, (struct sockaddr *)&sa, l) < 0) {
		PERROR("socket_init:bind"); 
		close(s);
		return(-1);
	}
	
	/* max # (1) of queued connects */
	listen(s, 1);

	getsockname(s, (struct sockaddr *)&sa, &l);
	fprintf(stderr,"Wait for connect in port %d\n",
		ntohs(sa.sin_port));

	return s;
}


int socket_wait(int s) /* s: socket created with establish() */
{
	struct sockaddr_in isa;
	char connect_from[80];

	int addrlen, t;
	//int ns;
	
	addrlen=sizeof(isa); /* find socket's address for accept() */
	getsockname(s, (struct  sockaddr  *)&isa, &addrlen);
	t = accept(s, (struct sockaddr *)&isa, &addrlen);
	if (t < 0) { /* accept connection if there is one */
		PERROR("socket_server_read:accept");
		return -1;
	}
	sprintf(connect_from,"Connect from %s [%d]",
		inet_ntoa(isa.sin_addr), 
		ntohs(isa.sin_port ));
	fprintf(stderr,"%s\n", connect_from);
	
	return t;
}

